/*
 Library for TSIC digital Temperature Sensor Type 206/306 and may more.
 using ZACwire Communication Protocol for the TSic
 Manufatorer:		IST AG, Hygrosens, ZMD
 Original Code:	http://www.mikrocontroller.net/topic/82087
 rewritten by:		Rolf Wagner
 Date:				2013.03.26
 Temperature-sensor uses 2 Pins, one for VCC and the second for Signal (= ZACwire Protocol).
 With call of the Function ".....getTSicTemp();" the Senor will get power measure and will be turned OFF.
 In that case it uses only energy while measuring. There is another method with Interrupts, measuring
 at an constant rate, which is not implemented jet.
 */
#include "tsic.h"

void TSIC_INIT(void) {
	TSIC_PORT_DDR_Power |= TSCI_POWER_PIN;		// Ausgang
	TSIC_PORT_DDR &= ~TSIC_SIGNAL_PIN;	// Eingang
}

void readSens(uint8_t *temp_value) {
	while (TSIC_SIGNAL_HIGH)
		; // wait until start bit starts

	while (TSIC_SIGNAL_LOW)
		;	// wait, TStrobe

	// read 7 data bits
	for (int i = 0; i < 8; i++) {
		while (TSIC_SIGNAL_HIGH)
			;              // wait for falling edge
		_delay_us(60);
		if (TSIC_SIGNAL_HIGH)
			*temp_value |= 1;
		*temp_value <<= 1;
		while (TSIC_SIGNAL_LOW)
			;           // wait until line comes high again
	}

}
uint8_t readParity() {
	while (TSIC_SIGNAL_HIGH)
		;             // wait for falling edge
	_delay_us(60);
	if (TSIC_SIGNAL_HIGH)
		return 1;
	while (TSIC_SIGNAL_LOW)
		;           // wait until line comes high again
	return 0;
}

uint8_t checkParity(uint8_t temp_value, uint8_t parityadd) {
	uint8_t parity = 0;
	uint8_t mask = 0b00000001;
	for (uint8_t i = 0; i < 8; i++) {
		if (temp_value & mask)
			parity++;
		mask <<= 1;
	}
	parity += parityadd;
	return parity & 1; // AND 1 entspricht MOD 2
}
uint8_t getTSicTemp(uint16_t *temp_value16) {
	uint8_t temp_value1 = 0;
	uint8_t temp_value2 = 0;
	uint8_t paritybits;

	TSIC_ON();
	_delay_us(60);  // wait for stabilization

//first byte
	readSens(&temp_value1);
	paritybits = readParity();

// second byte
	readSens(&temp_value2);
	paritybits |= (readParity() << 1);

	TSIC_OFF();

	if (checkParity(temp_value1, paritybits & 1))
		return 0;
	if (checkParity(temp_value2, paritybits & 2))
		return 0;

	*temp_value16 = (temp_value1 << 7) | temp_value2;

	return 1;                       // parity is OK
}
